#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>


#define SURDELTAT 1.0
#define B 2200.0 
#define C 31.9808315326491  
#define tildeT 39.191731155672   
#define M 1.0
#define A 0.392089377298999
#define Q 0.0990210257942779
#define XVBAR 0.067424052679620 
#define RHO1MIN 0.00374956833815908 
#define RHO1U 0.1699388 

//Parametres du calcul
#define PRECISION_ALPHA 300
#define PRECISION_T 401
#define MAX_T 40 
#define MAX_R 50000
#define DT 0.1
#define PRECISION_U 11
#define PRECISION_V 16

int valeursv[PRECISION_V] = {0,20,40,60,80,100,120,140,160,180,200,220,240,260,280,300};
float valeursdeltarho[PRECISION_V] = {0.0,0.00307643693222373,0.00615287386444746,0.00922931079667119,0.0123057477288949,0.0153821846611187,0.184648131670392,0.196726220081868,0.208804308493344,0.22088239690482,0.232960485316295,0.245038573727771,0.257116662139247,0.265877631317523,0.271321481262599,0.272192497253811};

float deltarho1(float v){
	 float deltarho;
	int i;


	if (v>0){
		deltarho = valeursdeltarho[PRECISION_V-1];
		for (i=0;i<PRECISION_V-1;i++){
			if ((valeursv[i]<=v)&&(v<valeursv[i+1])) {
					deltarho = (valeursdeltarho[i]*(valeursv[i+1]-v)+valeursdeltarho[i+1]*(v-valeursv[i]))/(valeursv[i+1]-valeursv[i]);
					break;
			}  	
		}       
	}
	else deltarho = 0;
	return deltarho;
}

float dv1(float v){
	 float dv;
         if (v>80) dv = 232.5365*v - 0.8880617*v*v + 0.0015029*v*v*v - 5564.097;
	else dv =0;
	return dv;
}

float dbarv1(float v){
	 float dbarv;
         dbarv = dv1(v)/3+dv1(v-50)/3+dv1(v-100)/3;
	return dbarv;
}

float dv0(float v){
	 float dv;
         if (v>0) dv = 188.3733*v - 0.635664*v*v + 0.0010903*v*v*v + 2417.68;
	else dv=0;
	return dv;
}

float dbarv0(float v){
	 float dbarv;
         dbarv = dv0(v)/3+dv0(v-50)/3+dv0(v-100)/3;
	return dbarv;
}


float DamageCurve(float v,float alpha){
    float damage;
    damage = dbarv1(v)*alpha+dbarv0(v)*(1-alpha);
    return damage;
}
int successor2D(float state[2],float u,float v,float dt){
         float nextstate[2];
         int i;
         if (v>0){
		nextstate[0] = 0;
		nextstate[1] = state[1]+ deltarho1(v)*(1-state[1]);
	}else{
		nextstate[0] = state[0]+dt;
                if ((tildeT-nextstate[0])>0){
			nextstate[1] = state[1]+dt*SURDELTAT/M*((-1)*(Q+XVBAR)*state[1]+(1+A*state[1])*(1-state[1])*(u+RHO1MIN));
		}
		else{
			nextstate[1] = state[1]+dt*SURDELTAT/M*((-1)*Q*state[1]+(1+A*state[1])*(1-state[1])*(u+RHO1MIN));
		}
	}
	for(i=0;i<2;i++){state[i]=nextstate[i];}
	return 0;
}

float WelfareVariation(float alpha,float u,float v,float dt){
         float r;
         if (v>0){
		r=(-1)*DamageCurve(v,alpha);
	}else{

		r = dt*SURDELTAT*(B-C*u);
	}
        return r;
}

float predecessor1D(float state[2],float next_r,float u,float v,float dt){
         float r;
	r = next_r-WelfareVariation(state[1],u,v,dt);
        return r;
}


int successor(float state[3],float u,float v,float dt){
        float r,nextstate2D[2];
	nextstate2D[0] = state[0];
	nextstate2D[1] = state[1];
        successor2D(nextstate2D,u,v,dt);
        state[2]= state[2]+WelfareVariation(state[1],u,v,dt);
	state[0] = nextstate2D[0];
	state[1] = nextstate2D[1];
	return 0;
 }

int IntegersToState(float state[2],int j, int k){
	state[0]= (float)(j)*MAX_T/(PRECISION_T-1);
	state[1]= (float)(k)/(PRECISION_ALPHA-1);
	return 0;
}
float barycentre(float x,float y,float a,float b, float c, float d){
	float bary;
	bary = (1-x)*(1-y)*a+x*(1-y)*b+(1-x)*y*c +x*y*d;
	return bary;

}
float rValueContruction(float r_values[PRECISION_T][PRECISION_ALPHA],float t,float alpha){
	float r,a,b,c,d,tintfloat,alphaintfloat,x,y;
        int tint,alphaint;
	if (t >= MAX_T){
		t = MAX_T;
	}
	tint = (int)(t*(PRECISION_T-1)/MAX_T);
	alphaint = (int)(alpha*(PRECISION_ALPHA-1));
	tintfloat = (float)(tint) * MAX_T/(PRECISION_T-1);
	alphaintfloat = (float)(alphaint) /(PRECISION_ALPHA-1);

	a=r_values[tint][alphaint];
	b=0;
	c=0;
	d=0;
	if (tint<PRECISION_T-1){
		b = r_values[tint+1][alphaint];
		if (alphaint<PRECISION_ALPHA-1){
			c = r_values[tint][alphaint+1];
			d = r_values[tint+1][alphaint+1];
		}
		else{
			c = r_values[tint][alphaint];
			d = r_values[tint+1][alphaint];
		}

	} else{
		b = r_values[tint][alphaint];
		if (alphaint<PRECISION_ALPHA-1){
			c = r_values[tint][alphaint+1];
			d = r_values[tint][alphaint+1];
		}
		else{
			c = r_values[tint][alphaint];
			d = r_values[tint][alphaint];
		}

	}
	x= (t-tintfloat)*(PRECISION_T-1)/MAX_T;
	y=(alpha-alphaintfloat)*(PRECISION_ALPHA-1);
	r = barycentre(x,y,a,b,c,d);
	return r;
}
int mainbis(int argc, char *argv[]){
	int i;
	float state[3],state2D[2],u,v,r;
	v = 0;
	u=0;
        for (i=1;i<10;i++){
	        state[0]=i;
		state[1]=(float)(i)/10;
        	state[2]= (float)(i)*(1+((float)(1))/100);
		printf("state[2] avant %.10f \n",state[2]);
		successor(state,u,v,1);
		state2D[0]=state[0];
		state2D[1]=state[1];
		printf("state[0] %f\n",state[0]);
		printf("state[1] %f\n",state[1]);
		printf("state[2] %f\n",state[2]);

		r=predecessor1D(state2D,state[2],u,v,1);
		printf("r %f\n",r);

	}

}


int WriteParamValues(FILE *f,int vmax,int tmin){
	int i;
        fprintf(f,"{\n");
        fprintf(f,"\"DELTAT\":%f,\n",1/SURDELTAT);
        fprintf(f,"\"T\":%f,\n",tildeT);
        fprintf(f,"\"A\":%f,\n",A);
        fprintf(f,"\"M\":%f,\n",M);
        fprintf(f,"\"Q\":%f,\n",Q);
        fprintf(f,"\"XVBAR\":%f,\n",XVBAR);
        fprintf(f,"\"B\":%f,\n",B);
        fprintf(f,"\"C\":%f,\n",C);
	fprintf(f,"\"RHO1MIN\":%f,\n",RHO1MIN);
        fprintf(f,"\"RHO1U\":%f,\n",RHO1U);
        fprintf(f,"\"VMAX\":%d,\n",vmax);
        fprintf(f,"\"TMIN\":%d,\n",tmin);
        fprintf(f,"\"PRECISION_ALPHA\":%d,\n",PRECISION_ALPHA);
        fprintf(f,"\"PRECISION_T\":%d,\n",PRECISION_T);
        fprintf(f,"\"MAX_T\":%d,\n",MAX_T);
        fprintf(f,"\"MAX_R\":%d,\n",MAX_R);
        fprintf(f,"\"DT\":%f,\n",DT);
        fprintf(f,"\"PRECISION_U\":%d,\n",PRECISION_U);
        fprintf(f,"\"PRECISION_V\":%d\n",PRECISION_V);
	fprintf(f,"},\n");

	return 0;
	}

int main(int argc, char *argv[])
{
	float dr,r,r_values[PRECISION_T][PRECISION_ALPHA],u_values[PRECISION_T][PRECISION_ALPHA],ucourant,UMAX,state[2],nextstate[2],mini,maxi,u,v,inter,state3D[3],nextstate3D[3],newdt;
	int i,j,k,l,b,bb,nbpassage,NT,tmax,vmax,nextj,nextk,actualj,actualk,nbtraj,NBtraj,degraderecord,degraderecordr,degraderecordu,vmaxcommand,tmincommand,umaxcommand;
	FILE *f;

	if ((argc>=5)&&(strcmp(argv[1], "-vmax") == 0)&&(strcmp(argv[3], "-tmin") == 0)){
		tmincommand = atoi(argv[4]);
		vmaxcommand = atoi(argv[2]);

	        degraderecord = 1; //1 pas de degradation
	        degraderecordr = 1; //1 pas de degradation
	        degraderecordu = 1; //1 pas de degradation

		f=fopen("json_data_RAZ13_2200.txt","w");
	        fprintf(f,"[\n");
	        WriteParamValues(f,vmaxcommand,tmincommand);
		newdt = (float)(DT)/10;
	        NBtraj = 1;
		vmax=vmaxcommand;
		tmax = tmincommand;
		UMAX = RHO1U-RHO1MIN;
	        fprintf(f,"{\n");
	        nbpassage = 0;
		for(j=PRECISION_T-1;j>=0;j--){
			for(k=0;k<PRECISION_ALPHA;k++){
				u_values[j][k] = 0;
				r_values[j][k] = 0;
	        	}
	        }
		b = 1;
		bb = 1;
		while((b == 1)&&(bb == 1)){
			b = 0;
		        bb = 0;
			for(j=PRECISION_T-1;j>=0;j--){
				for(k=0;k<PRECISION_ALPHA;k++){
					IntegersToState(state,j, k);
		                        mini = MAX_R; 
					ucourant = -2;
					if (state[0]>=tmax){
						for(i=0;i<PRECISION_U;i++){
							u = i*UMAX/(PRECISION_U-1);
							
							maxi = 0;
							for(l=0;l<PRECISION_V;l++){
								v = l*vmax/(PRECISION_V-1);

								nextstate[0] = state[0];
								nextstate[1] = state[1];
								successor2D(nextstate,u,v,DT);
	                                        		if (nextstate[0] > MAX_T) {
									nextstate[0] = MAX_T;

								}
								
                                                		inter = predecessor1D(state,rValueContruction(r_values,nextstate[0],nextstate[1]),u,v,DT);
								if (inter > maxi) {maxi = inter;} 
							}	
							if (mini > maxi) {mini = maxi;ucourant=u;} 
						}
					}
					else{
						for(i=0;i<PRECISION_U;i++){
							u = i*UMAX/(PRECISION_U-1);
							nextstate[0] = state[0];
							nextstate[1] = state[1];
							successor2D(nextstate,u,0,DT);
		                                        if (nextstate[0] > MAX_T) {nextstate[0] = MAX_T;}
							inter = predecessor1D(state,rValueContruction(r_values,nextstate[0],nextstate[1]),u,0,DT);
							if (mini > inter) {mini = inter;ucourant=u;} 
						} 
				        }
					if (r_values[j][k] < mini) {
						r_values[j][k] = mini;
						u_values[j][k] = ucourant;
						b = 1;
					} 
					if (j == 0) {
						if (r_values[j][k] == 0){ bb = 1;}
					} 

			        }
		        }
		        nbpassage = nbpassage +1;
       		printf("nbpassage = %d, tmax = %d, vmax = %d,b= %d, bb = %d",nbpassage,tmax,vmax,b,bb);
    		}
     		if (bb == 0){
			for(j=0;j<PRECISION_T;j++){
				for(k=0;k<PRECISION_ALPHA;k++){
					r_values[j][k] = -1;
					u_values[j][k] = -1;
				}
			}
     		}
		fprintf(f,"\"x\":[");
		for(j=0;j<PRECISION_T;j++){
			if(j % degraderecordr ==0){
				if (j<PRECISION_T-degraderecordr) {fprintf(f,"%.2f,",(float)(j)*MAX_T/(PRECISION_T-1));}
                		else {fprintf(f,"%.2f",(float)(j)*MAX_T/(PRECISION_T-1));}
			}        	
		}
		fprintf(f,"],\n");
		fprintf(f,"\"y\":[");
		for(k=0;k<PRECISION_ALPHA;k++){
			if(k % degraderecordr ==0){
				if (k<PRECISION_ALPHA-degraderecordr) {fprintf(f,"%.5f,",(float)(k)/(PRECISION_ALPHA-1));}
                		else {fprintf(f,"%.2f",(float)(k)/(PRECISION_ALPHA-1));}
			}
	        }
	       fprintf(f,"],\n");
		fprintf(f,"\"z\":[\n");
		if (bb !=0){
			for(k=0;k<PRECISION_ALPHA;k++){
				if(k % degraderecordr ==0){
					fprintf(f,"[");
					for(j=0;j<PRECISION_T;j++){
						if(j % degraderecordr ==0){
							if (j<PRECISION_T-degraderecordr) {fprintf(f,"%.2f,",r_values[j][k]);}
                					else {fprintf(f,"%.2f",r_values[j][k]);}
        					}
					}
					if (k<PRECISION_ALPHA-degraderecordr) {fprintf(f,"],\n");}
                			else {fprintf(f,"]\n");}
				}
        		}
		}
	       fprintf(f,"],\n");

		fprintf(f,"\"xu\":[");
		for(j=0;j<PRECISION_T;j++){
			if(j % degraderecordu ==0){
				if (j<PRECISION_T-degraderecordu) {fprintf(f,"%.2f,",(float)(j)*MAX_T/(PRECISION_T-1));}
                		else {fprintf(f,"%.2f",(float)(j)*MAX_T/(PRECISION_T-1));}
			}        	
		}
       		fprintf(f,"],\n");
		fprintf(f,"\"yu\":[");
		for(k=0;k<PRECISION_ALPHA;k++){
			if(k % degraderecordu ==0){
				if (k<PRECISION_ALPHA-degraderecordu) {fprintf(f,"%.5f,",(float)(k)/(PRECISION_ALPHA-1));}
                		else {fprintf(f,"%.2f",(float)(k)/(PRECISION_ALPHA-1));}
			}
        	}
	       fprintf(f,"],\n");
		fprintf(f,"\"u\":[\n");
		if (bb !=0){// 
			for(k=0;k<PRECISION_ALPHA;k++){
				if(k % degraderecordu ==0){
					fprintf(f,"[");
					for(j=0;j<PRECISION_T;j++){
						if(j % degraderecordu ==0){
							if (j<PRECISION_T-degraderecordu) {fprintf(f,"%.2f,",u_values[j][k]);}//r
                					else {fprintf(f,"%.2f",u_values[j][k]);}//r
        					}
					}
					if (k<PRECISION_ALPHA-degraderecordu) {fprintf(f,"],\n");}
                			else {fprintf(f,"]\n");}
				}
		        }
		}
       		fprintf(f,"],\n");

		NT = (int)(tmax*10/newdt);
	        for (nbtraj=0;nbtraj<NBtraj;nbtraj++){
		       state3D[0] = 0;
		        state3D[1] = 0+1*nbtraj;
		        state3D[2] = r_values[0][0+nbtraj*(PRECISION_ALPHA-1)]+20;
		        if (r_values[0][0]>=0){
				printf("dedans\n");
				fprintf(f,"\"traj%d\":[\n",nbtraj);

				for(j=0;j<NT;j++){
        	               		 b = 0;
        	               		 v = 0;
        	               		 if (nbtraj == 0){
		               		         if ((j>0)&&(j % ((int)(NT/10)+20) == 0)){v = vmax;}
					}
					else if (nbtraj == 1){
	
		               		         if ((j>0)&&(j % ((int)(NT/10)+40) == 0)&&(j % ((int)(4*NT/10)+40) > (int)(2*NT/10)+40)){v = vmax;printf("maintenant %d ",j);}
		
					}
					actualj = (int)(state3D[0]*(PRECISION_T-1)/MAX_T+0.0);
					actualk = (int)(state3D[1]*(PRECISION_ALPHA-1)+0.0);
					IntegersToState(state,actualj,actualk);
				


					if (rValueContruction(r_values,state3D[0],state3D[1])+10<state3D[2]){
						u =0;
					}
					else if(rValueContruction(r_values,state3D[0],state3D[1])<state3D[2]){
						u=u_values[actualj][actualk];
					}
					else {
					}
					
					successor(state3D,u,v,newdt);
	                	        if (state3D[0] > MAX_T) {state3D[0] = MAX_T;}
					if(j % degraderecord ==0){
						if (j<NT-degraderecord) {fprintf(f,"[%.2f,%.2f,%.6f,%.2f,%.2f,%d,%d,%.2f,%.2f,%.2f],\n",(float)(j)*newdt,state3D[0],state3D[1],state3D[2],rValueContruction(r_values,state3D[0],state3D[1]),actualj,actualk,r_values[actualj][actualk],u,v);}
					        else {fprintf(f,"[%.2f,%.2f,%.6f,%.2f,%.2f,%d,%d,%.2f,%.2f,%.2f],\n",(float)(j)*newdt,state3D[0],state3D[1],state3D[2],rValueContruction(r_values,state3D[0],state3D[1]),actualj,actualk,r_values[actualj][actualk],u,v);}
					}
					b=1;
                        		if (b ==0 ) {printf("problem");
		        		      fprintf(f,"[-1,-1,-1,-1,-1,-1]\n");
						break;
					}				
				}

			}
        		else{
				fprintf(f,"\"traj%d\":[\n",nbtraj);
				fprintf(f,"[-1,-1,-1,-1,-1,-1]\n");
       			}       
       			if (nbtraj<(NBtraj-1)){fprintf(f,"],\n");}
			else {fprintf(f,"]\n");}

		}
	       fprintf(f,"}");
	       fprintf(f,"\n]");
		  fclose(f);
	}
        else{
          printf("Incorrect arguments -vmax (int) -tmin (int)\n");
        }

   return 0;
 }
